@@ -1,5 +1,6 @@
 import os
-
+import re
+import pandas as pd
 from metrics_layer.core.model.definitions import Definitions
 
 
@@ -76,6 +77,22 @@ class SeedMetricsLayer:
             "DECIMAL": "number",
             "BIGINT": "number",
         }
+        self._druid_type_lookup = {
+            "CHAR": "string",
+            "VARCHAR": "string",
+            "DECIMAL": "number",
+            "FLOAT": "number",
+            "REAL": "number",
+            "DOUBLE": "number",
+            "BOOLEAN": "yesno",
+            "TINYINT": "number",
+            "SMALLINT": "number",
+            "INTEGER": "number",
+            "BIGINT": "number",
+            "TIMESTAMP": "timestamp",
+            "DATE": "date",
+        }
+        # Also duck db type lookup
         self._postgres_type_lookup = {
             "date": "date",
             "timestamp without time zone": "timestamp",
@@ -92,6 +109,27 @@ class SeedMetricsLayer:
             "real": "number",
             "money": "number",
         }
+        self._sql_server_type_lookup = {
+            "float": "number",
+            "real": "number",
+            "bigint": "number",
+            "numeric": "number",
+            "smallint": "number",
+            "decimal": "number",
+            "smallmoney": "number",
+            "int": "number",
+            "tinyint": "number",
+            "money": "number",
+            "char": "string",
+            "varchar": "string",
+            "text": "string",
+            "date": "date",
+            "datetime2": "datetime",
+            "smalldatetime": "datetime",
+            "datetime": "datetime",
+            "time": "datetime",
+            "bit": "yesno",
+        }
         self._bigquery_type_lookup = {
             "DATE": "date",
             "DATETIME": "timestamp",
@@ -113,15 +151,25 @@ class SeedMetricsLayer:
         columns_query = self.columns_query()
         data = self.run_query(columns_query)
         data.columns = [c.upper() for c in data.columns]
+
+        if self.connection.type == Definitions.snowflake:
+            table_query = self.table_query()
+            table_data = self.run_query(table_query)
+            table_data.columns = [c.upper() for c in table_data.columns]
+        else:
+            table_data = pd.DataFrame()
         print(f"Got information on all tables and views in {self.location_description}")
 
         # We need to filter down the whole result to just the chosen schema and table (if applicable)
         if self.schema:
             data = data[data["TABLE_SCHEMA"].str.lower() == self.schema.lower()].copy()
+            if not table_data.empty:
+                table_data = table_data[table_data["TABLE_SCHEMA"].str.lower() == self.schema.lower()].copy()
 
         if self.table:
             data = data[data["TABLE_NAME"].str.lower() == self.table.lower()].copy()
-
+            if not table_data.empty:
+                table_data = table_data[table_data["TABLE_NAME"].str.lower() == self.table.lower()].copy()
         folder = self._location()
         loader = ProjectLoader(folder)
         project = loader.load()
@@ -133,9 +181,14 @@ class SeedMetricsLayer:
         views, tables = [], data["TABLE_NAME"].unique()
         for i, table_name in enumerate(tables):
             column_df = data[data["TABLE_NAME"].str.lower() == table_name.lower()].copy()
+            if not table_data.empty:
+                table_comment = table_data[table_data["TABLE_NAME"].str.lower() == table_name.lower()][
+                    "COMMENT"
+                ].values[0]
+            else:
+                table_comment = None
             schema_name = column_df["TABLE_SCHEMA"].values[0]
-            view = self.make_view(column_df, model_name, table_name, schema_name)
-
+            view = self.make_view(column_df, model_name, table_name, schema_name, table_comment)
             if self.table:
                 progress = ""
             else:
@@ -181,14 +234,23 @@ class SeedMetricsLayer:
         }
         return [model]
 
-    def make_view(self, column_data, model_name: str, table_name: str, schema_name: str):
+    def make_view(
+        self, column_data, model_name: str, table_name: str, schema_name: str, table_comment: str = None
+    ):
         view_name = self.clean_name(table_name)
-        count_measure = {"field_type": "measure", "name": "count", "type": "count"}
-        fields = self.make_fields(column_data) + [count_measure]
-        if self.connection.type in {Definitions.snowflake, Definitions.redshift, Definitions.postgres}:
+        fields = self.make_fields(column_data)
+        if self.connection.type in {
+            Definitions.snowflake,
+            Definitions.redshift,
+            Definitions.postgres,
+            Definitions.sql_server,
+            Definitions.duck_db,
+        }:
             sql_table_name = f"{schema_name}.{table_name}"
             if self._database_is_not_default:
                 sql_table_name = f"{self.database}.{sql_table_name}"
+        elif self.connection.type == Definitions.druid:
+            sql_table_name = f"{schema_name}.{table_name}"
         elif self.connection.type == Definitions.bigquery:
             sql_table_name = f"`{self.database}.{schema_name}.{table_name}`"
         else:
@@ -200,32 +262,44 @@ class SeedMetricsLayer:
             "model_name": model_name,
             "sql_table_name": sql_table_name,
             "default_date": next((f["name"] for f in fields if f["field_type"] == "dimension_group"), None),
-            "row_label": "TODO - Label row",
             "fields": fields,
         }
+        if table_comment:
+            view["description"] = table_comment
         if view["default_date"] is None:
             view.pop("default_date")
         return view
 
     def make_fields(self, column_data: str):
         fields = []
-        for _, row in column_data[["COLUMN_NAME", "DATA_TYPE"]].iterrows():
+        if self.connection.type == Definitions.snowflake:
+            data_to_iterate = column_data[["COLUMN_NAME", "DATA_TYPE", "COMMENT"]]
+        else:
+            data_to_iterate = column_data[["COLUMN_NAME", "DATA_TYPE"]]
+        for _, row in data_to_iterate.iterrows():
             name = self.clean_name(row["COLUMN_NAME"])
             if self.connection.type == Definitions.snowflake:
                 metrics_layer_type = self._snowflake_type_lookup.get(row["DATA_TYPE"], "string")
             elif self.connection.type == Definitions.redshift:
                 metrics_layer_type = self._redshift_type_lookup.get(row["DATA_TYPE"], "string")
-            elif self.connection.type == Definitions.postgres:
+            elif self.connection.type in {Definitions.postgres, Definitions.duck_db}:
                 metrics_layer_type = self._postgres_type_lookup.get(row["DATA_TYPE"], "string")
             elif self.connection.type == Definitions.bigquery:
                 metrics_layer_type = self._bigquery_type_lookup.get(row["DATA_TYPE"], "string")
+            elif self.connection.type == Definitions.druid:
+                metrics_layer_type = self._druid_type_lookup.get(row["DATA_TYPE"], "string")
+            elif self.connection.type == Definitions.sql_server:
+                metrics_layer_type = self._sql_server_type_lookup.get(row["DATA_TYPE"], "string")
             else:
                 raise NotImplementedError(f"Unknown connection type: {self.connection.type}")
             sql = "${TABLE}." + row["COLUMN_NAME"]
 
             field = {"name": name, "sql": sql}
 
-            if metrics_layer_type in {"timestamp", "date"}:
+            if "COMMENT" in row and row["COMMENT"] is not None:
+                field["description"] = row["COMMENT"]
+
+            if metrics_layer_type in {"timestamp", "date", "datetime"}:
                 field["field_type"] = "dimension_group"
                 field["type"] = "time"
                 field["timeframes"] = ["raw", "date", "week", "month", "quarter", "year"]
@@ -237,13 +311,23 @@ class SeedMetricsLayer:
         return fields
 
     def columns_query(self):
-        query = "SELECT table_catalog, table_schema, table_name, column_name, data_type FROM "
+        comment_statement = ", comment as comment" if self.connection.type == Definitions.snowflake else ""
+        query = (
+            f"SELECT table_catalog, table_schema, table_name, column_name, data_type{comment_statement} FROM "
+        )
         if self.database and self.connection.type in {
             Definitions.snowflake,
             Definitions.redshift,
             Definitions.postgres,
+            Definitions.sql_server,
+            Definitions.duck_db,
         }:
             query += f"{self.database}.INFORMATION_SCHEMA.COLUMNS"
+        elif self.connection.type == Definitions.druid:
+            query = (
+                "SELECT TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, COLUMN_NAME, DATA_TYPE "
+                "FROM INFORMATION_SCHEMA.COLUMNS"
+            )
         elif self.database and self.schema and self.connection.type == Definitions.bigquery:
             query += f"`{self.database}.{self.schema}`.INFORMATION_SCHEMA.COLUMNS"
         elif not self.schema and self.connection.type == Definitions.bigquery:
@@ -252,7 +336,7 @@ class SeedMetricsLayer:
             )
         else:
             raise ValueError("You must specify at least a database for seeding")
-        return query + ";"
+        return query + ";" if self.connection.type != Definitions.druid else query
 
     def table_query(self):
         if self.database and self.connection.type == Definitions.snowflake:
@@ -260,9 +344,21 @@ class SeedMetricsLayer:
                 "SELECT table_catalog as table_database, table_schema as table_schema, "
                 "table_name as table_name, table_owner as table_owner, table_type as table_type, "
                 "bytes as table_size, created as table_created, last_altered as table_last_modified, "
-                f"row_count as table_row_count FROM {self.database}.INFORMATION_SCHEMA.TABLES"
+                f"row_count as table_row_count, comment as comment FROM {self.database}.INFORMATION_SCHEMA.TABLES"
             )
-        elif self.database and self.connection.type in {Definitions.redshift, Definitions.postgres}:
+        elif self.connection.type in {Definitions.druid}:
+            query = (
+                "SELECT TABLE_CATALOG as table_database, TABLE_SCHEMA as table_schema, "
+                "TABLE_NAME as table_name, TABLE_TYPE as table_type "
+                "FROM INFORMATION_SCHEMA.TABLES "
+                "WHERE TABLE_SCHEMA not in ('sys', 'INFORMATION_SCHEMA')"
+            )
+        elif self.database and self.connection.type in {
+            Definitions.redshift,
+            Definitions.postgres,
+            Definitions.sql_server,
+            Definitions.duck_db,
+        }:
             query = (
                 "SELECT table_catalog as table_database, table_schema as table_schema, "
                 "table_name as table_name, table_type as table_type "
@@ -282,7 +378,7 @@ class SeedMetricsLayer:
             )
         else:
             raise ValueError("You must specify at least a database for seeding")
-        return query + ";"
+        return query + ";" if self.connection.type != Definitions.druid else query
 
     def run_query(self, query: str):
         if self.run_query_override:
@@ -401,7 +497,8 @@ class SeedMetricsLayer:
 
     @staticmethod
     def clean_name(txt: str):
-        return txt.lower().replace(" ", "_")
+        alphanumeric_string = re.sub(r"[^a-zA-Z0-9\s_]", "", txt)
+        return alphanumeric_string.lower().strip().replace(" ", "_")
 
 
 class dbtSeed(SeedMetricsLayer):
@@ -460,18 +557,8 @@ class dbtSeed(SeedMetricsLayer):
         default_date = next(
             (f["name"] for f in fields if f.get("meta", {}).get("datatype") is not None), None
         )
-        count_metric = {
-            "name": "count",
-            "label": "Count",
-            "calculation_method": "count",
-            "expression": fields[0]["name"],
-            "model": f"ref('{view_name}')",
-            "description": "The count of the rows in the table",
-            "timestamp": default_date,
-            "time_grains": ["day", "week", "month", "quarter", "year"],
-        }
-        metrics = [count_metric] if default_date is not None else []
-        model_meta = {"row_label": "TODO - Label row", "default_date": default_date, "identifiers": []}
+        metrics = []
+        model_meta = {"default_date": default_date, "identifiers": []}
         if model_meta["default_date"] is None:
             model_meta.pop("default_date")
 
@@ -491,7 +578,7 @@ class dbtSeed(SeedMetricsLayer):
                 metrics_layer_type = self._snowflake_type_lookup.get(row["DATA_TYPE"], "string")
             elif self.connection.type == Definitions.redshift:
                 metrics_layer_type = self._redshift_type_lookup.get(row["DATA_TYPE"], "string")
-            elif self.connection.type == Definitions.postgres:
+            elif self.connection.type in {Definitions.postgres, Definitions.duck_db}:
                 metrics_layer_type = self._postgres_type_lookup.get(row["DATA_TYPE"], "string")
             elif self.connection.type == Definitions.bigquery:
                 metrics_layer_type = self._bigquery_type_lookup.get(row["DATA_TYPE"], "string")
