@@ -10,6 +10,36 @@ from .filter import Filter
 from .set import Set
 
 SQL_KEYWORDS = {"order", "group", "by", "as", "from", "select", "on", "with"}
+VALID_TIMEFRAMES = [
+    "raw",
+    "time",
+    "second",
+    "minute",
+    "hour",
+    "date",
+    "week",
+    "month",
+    "quarter",
+    "year",
+    "week_index",
+    "month_of_year",
+    "month_of_year_index",
+    "month_name",
+    "month_index",
+    "hour_of_day",
+    "day_of_week",
+    "day_of_month",
+]
+VALID_INTERVALS = [
+    "second",
+    "minute",
+    "hour",
+    "day",
+    "week",
+    "month",
+    "quarter",
+    "year",
+]
 
 
 class Field(MetricsLayerBase, SQLReplacement):
@@ -109,7 +139,15 @@ class Field(MetricsLayerBase, SQLReplacement):
 
     @property
     def convert_timezone(self):
-        return self._definition.get("convert_timezone", True) and self._definition.get("convert_tz", True)
+        default_value = True
+        if self.view.model.default_convert_tz is False or self.view.model.default_convert_timezone is False:
+            default_value = False
+
+        if "convert_timezone" in self._definition:
+            convert = self._definition.get("convert_timezone", default_value)
+        else:
+            convert = self._definition.get("convert_tz", default_value)
+        return convert
 
     @property
     def datatype(self):
@@ -128,7 +166,7 @@ class Field(MetricsLayerBase, SQLReplacement):
     @property
     def canon_date(self):
         if "canon_date" in self._definition:
-            canon_date = self._definition["canon_date"]
+            canon_date = self._definition["canon_date"].replace("${", "").replace("}", "")
             return self._add_view_name_if_needed(canon_date)
         if self.view.default_date:
             return self._add_view_name_if_needed(self.view.default_date)
@@ -184,7 +222,7 @@ class Field(MetricsLayerBase, SQLReplacement):
         return self.get_replaced_sql_query(query_type, alias_only=alias_only)
 
     def aggregate_sql_query(self, query_type: str, functional_pk: str, alias_only: bool = False):
-        # TODO add median, median_distinct, percentile, max, min, percentile, percentile_distinct
+        # TODO add median_distinct, percentile, percentile, percentile_distinct
         sql = self.raw_sql_query(query_type, alias_only=alias_only)
         type_lookup = {
             "sum": self._sum_aggregate_sql,
@@ -194,8 +232,14 @@ class Field(MetricsLayerBase, SQLReplacement):
             "average": self._average_aggregate_sql,
             "average_distinct": self._average_distinct_aggregate_sql,
             "median": self._median_aggregate_sql,
+            "max": self._max_aggregate_sql,
+            "min": self._min_aggregate_sql,
             "number": self._number_aggregate_sql,
         }
+        if self.type not in type_lookup:
+            raise QueryError(
+                f"Aggregate type {self.type} not supported. Supported types are: {list(type_lookup.keys())}"
+            )
         return type_lookup[self.type](sql, query_type, functional_pk, alias_only)
 
     def strict_replaced_query(self):
@@ -241,7 +285,7 @@ class Field(MetricsLayerBase, SQLReplacement):
 
     def _sum_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
         # Druid doesn't support symmetric aggregates, so we have to ignore this check
-        no_symmetric_aggregates = {Definitions.druid}
+        no_symmetric_aggregates = {Definitions.druid, Definitions.sql_server}
         if self._needs_symmetric_aggregate(functional_pk) and query_type not in no_symmetric_aggregates:
             return self._sum_symmetric_aggregate(sql, query_type, alias_only=alias_only)
         return f"SUM({sql})"
@@ -262,7 +306,7 @@ class Field(MetricsLayerBase, SQLReplacement):
         alias_only: bool = False,
         factor: int = 1_000_000,
     ):
-        if query_type in {Definitions.druid}:
+        if query_type in {Definitions.druid, Definitions.sql_server}:
             raise QueryError(
                 "Symmetric aggregates are not supported in Druid. "
                 "Use the 'sum' type instead of 'sum_distinct'."
@@ -315,7 +359,7 @@ class Field(MetricsLayerBase, SQLReplacement):
         return result
 
     def _count_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
-        no_symmetric_aggregates = {Definitions.druid}
+        no_symmetric_aggregates = {Definitions.druid, Definitions.sql_server}
         if self._needs_symmetric_aggregate(functional_pk) and query_type not in no_symmetric_aggregates:
             if self.primary_key_count:
                 return self._count_distinct_aggregate_sql(
@@ -348,7 +392,7 @@ class Field(MetricsLayerBase, SQLReplacement):
         return result
 
     def _average_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
-        no_symmetric_aggregates = {Definitions.druid}
+        no_symmetric_aggregates = {Definitions.druid, Definitions.sql_server}
         if self._needs_symmetric_aggregate(functional_pk) and query_type not in no_symmetric_aggregates:
             return self._average_symmetric_aggregate(sql, query_type, alias_only=alias_only)
         return f"AVG({sql})"
@@ -356,7 +400,7 @@ class Field(MetricsLayerBase, SQLReplacement):
     def _average_distinct_aggregate_sql(
         self, sql: str, query_type: str, functional_pk: str, alias_only: bool
     ):
-        if query_type in {Definitions.druid}:
+        if query_type in {Definitions.druid, Definitions.sql_server}:
             raise QueryError(
                 "Symmetric aggregates are not supported in Druid. "
                 "Use the 'average' type instead of 'average_distinct'."
@@ -379,7 +423,12 @@ class Field(MetricsLayerBase, SQLReplacement):
         return result
 
     def _median_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
-        if query_type in {Definitions.druid, Definitions.postgres, Definitions.bigquery}:
+        if query_type in {
+            Definitions.druid,
+            Definitions.postgres,
+            Definitions.bigquery,
+            Definitions.sql_server,
+        }:
             raise QueryError(
                 f"Median is not supported in {query_type}. Please choose another "
                 f"aggregate function for the {self.id()} measure."
@@ -387,6 +436,14 @@ class Field(MetricsLayerBase, SQLReplacement):
         # Medians do not work with symmetric aggregates, so there's just the one return
         return f"MEDIAN({sql})"
 
+    def _max_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
+        # Max works natively with symmetric aggregates, so there's just the one return
+        return f"MAX({sql})"
+
+    def _min_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
+        # Min works natively with symmetric aggregates, so there's just the one return
+        return f"MIN({sql})"
+
     def _number_aggregate_sql(self, sql: str, query_type: str, functional_pk: str, alias_only: bool):
         if isinstance(sql, list):
             replaced = deepcopy(self.sql)
@@ -526,6 +583,16 @@ class Field(MetricsLayerBase, SQLReplacement):
                 "quarters": lambda start, end: f"TIMESTAMPDIFF(QUARTER, {start}, {end})",
                 "years": lambda start, end: f"TIMESTAMPDIFF(YEAR, {start}, {end})",
             },
+            Definitions.sql_server: {
+                "seconds": lambda start, end: f"DATEDIFF(SECOND, {start}, {end})",
+                "minutes": lambda start, end: f"DATEDIFF(MINUTE, {start}, {end})",
+                "hours": lambda start, end: f"DATEDIFF(HOUR, {start}, {end})",
+                "days": lambda start, end: f"DATEDIFF(DAY, {start}, {end})",
+                "weeks": lambda start, end: f"DATEDIFF(WEEK, {start}, {end})",
+                "months": lambda start, end: f"DATEDIFF(MONTH, {start}, {end})",
+                "quarters": lambda start, end: f"DATEDIFF(QUARTER, {start}, {end})",
+                "years": lambda start, end: f"DATEDIFF(YEAR, {start}, {end})",
+            },
             Definitions.bigquery: {
                 "seconds": lambda start, end: f"TIMESTAMP_DIFF(CAST({end} as TIMESTAMP), CAST({start} as TIMESTAMP), SECOND)",  # noqa
                 "minutes": lambda start, end: f"TIMESTAMP_DIFF(CAST({end} as TIMESTAMP), CAST({start} as TIMESTAMP), MINUTE)",  # noqa
@@ -539,6 +606,8 @@ class Field(MetricsLayerBase, SQLReplacement):
         }
         # Snowflake and redshift have identical syntax in this case
         meta_lookup[Definitions.redshift] = meta_lookup[Definitions.snowflake]
+        # Snowflake and duck db have identical syntax in this case
+        meta_lookup[Definitions.duck_db] = meta_lookup[Definitions.snowflake]
         try:
             return meta_lookup[query_type][dimension_group](sql_start, sql_end)
         except KeyError:
@@ -554,11 +623,16 @@ class Field(MetricsLayerBase, SQLReplacement):
             Definitions.snowflake: {
                 "raw": lambda s, qt: s,
                 "time": lambda s, qt: f"CAST({s} AS TIMESTAMP)",
+                "second": lambda s, qt: f"DATE_TRUNC('SECOND', {s})",
+                "minute": lambda s, qt: f"DATE_TRUNC('MINUTE', {s})",
+                "hour": lambda s, qt: f"DATE_TRUNC('HOUR', {s})",
                 "date": lambda s, qt: f"DATE_TRUNC('DAY', {s})",
                 "week": self._week_dimension_group_time_sql,
                 "month": lambda s, qt: f"DATE_TRUNC('MONTH', {s})",
                 "quarter": lambda s, qt: f"DATE_TRUNC('QUARTER', {s})",
                 "year": lambda s, qt: f"DATE_TRUNC('YEAR', {s})",
+                "week_index": lambda s, qt: f"EXTRACT(WEEK FROM {s})",
+                "month_of_year_index": lambda s, qt: f"EXTRACT(MONTH FROM {s})",
                 "month_of_year": lambda s, qt: f"TO_CHAR(CAST({s} AS TIMESTAMP), 'MON')",
                 "hour_of_day": lambda s, qt: f"HOUR(CAST({s} AS TIMESTAMP))",
                 "day_of_week": lambda s, qt: f"TO_CHAR(CAST({s} AS TIMESTAMP), 'Dy')",
@@ -567,11 +641,16 @@ class Field(MetricsLayerBase, SQLReplacement):
             Definitions.postgres: {
                 "raw": lambda s, qt: s,
                 "time": lambda s, qt: f"CAST({s} AS TIMESTAMP)",
+                "second": lambda s, qt: f"DATE_TRUNC('SECOND', CAST({s} AS TIMESTAMP))",
+                "minute": lambda s, qt: f"DATE_TRUNC('MINUTE', CAST({s} AS TIMESTAMP))",
+                "hour": lambda s, qt: f"DATE_TRUNC('HOUR', CAST({s} AS TIMESTAMP))",
                 "date": lambda s, qt: f"DATE_TRUNC('DAY', CAST({s} AS TIMESTAMP))",
                 "week": self._week_dimension_group_time_sql,
                 "month": lambda s, qt: f"DATE_TRUNC('MONTH', CAST({s} AS TIMESTAMP))",
                 "quarter": lambda s, qt: f"DATE_TRUNC('QUARTER', CAST({s} AS TIMESTAMP))",
                 "year": lambda s, qt: f"DATE_TRUNC('YEAR', CAST({s} AS TIMESTAMP))",
+                "week_index": lambda s, qt: f"EXTRACT(WEEK FROM {s})",
+                "month_of_year_index": lambda s, qt: f"EXTRACT(MONTH FROM {s})",
                 "month_of_year": lambda s, qt: f"TO_CHAR(CAST({s} AS TIMESTAMP), 'MON')",
                 "hour_of_day": lambda s, qt: f"EXTRACT('HOUR' FROM CAST({s} AS TIMESTAMP))",
                 "day_of_week": lambda s, qt: f"TO_CHAR(CAST({s} AS TIMESTAMP), 'Dy')",
@@ -580,24 +659,52 @@ class Field(MetricsLayerBase, SQLReplacement):
             Definitions.druid: {
                 "raw": lambda s, qt: s,
                 "time": lambda s, qt: f"CAST({s} AS TIMESTAMP)",
+                "second": lambda s, qt: f"DATE_TRUNC('SECOND', CAST({s} AS TIMESTAMP))",
+                "minute": lambda s, qt: f"DATE_TRUNC('MINUTE', CAST({s} AS TIMESTAMP))",
+                "hour": lambda s, qt: f"DATE_TRUNC('HOUR', CAST({s} AS TIMESTAMP))",
                 "date": lambda s, qt: f"DATE_TRUNC('DAY', CAST({s} AS TIMESTAMP))",
                 "week": self._week_dimension_group_time_sql,
                 "month": lambda s, qt: f"DATE_TRUNC('MONTH', CAST({s} AS TIMESTAMP))",
                 "quarter": lambda s, qt: f"DATE_TRUNC('QUARTER', CAST({s} AS TIMESTAMP))",
                 "year": lambda s, qt: f"DATE_TRUNC('YEAR', CAST({s} AS TIMESTAMP))",
+                "week_index": lambda s, qt: f"EXTRACT(WEEK FROM {s})",
+                "month_of_year_index": lambda s, qt: f"EXTRACT(MONTH FROM {s})",
                 "month_of_year": lambda s, qt: f"CASE EXTRACT(MONTH FROM {s}) WHEN 1 THEN 'Jan' WHEN 2 THEN 'Feb' WHEN 3 THEN 'Mar' WHEN 4 THEN 'Apr' WHEN 5 THEN 'May' WHEN 6 THEN 'Jun' WHEN 7 THEN 'Jul' WHEN 8 THEN 'Aug' WHEN 9 THEN 'Sep' WHEN 10 THEN 'Oct' WHEN 11 THEN 'Nov' WHEN 12 THEN 'Dec' ELSE 'Invalid Month' END",  # noqa
                 "hour_of_day": lambda s, qt: f"EXTRACT(HOUR FROM CAST({s} AS TIMESTAMP))",
                 "day_of_week": lambda s, qt: f"CASE EXTRACT(DOW FROM {s}) WHEN 1 THEN 'Mon' WHEN 2 THEN 'Tue' WHEN 3 THEN 'Wed' WHEN 4 THEN 'Thu' WHEN 5 THEN 'Fri' WHEN 6 THEN 'Sat' WHEN 7 THEN 'Sun' ELSE 'Invalid Day' END",  # noqa
                 "day_of_month": lambda s, qt: f"EXTRACT(DAY FROM CAST({s} AS TIMESTAMP))",
             },
+            Definitions.sql_server: {
+                "raw": lambda s, qt: s,
+                "time": lambda s, qt: f"CAST({s} AS DATETIME)",
+                "second": lambda s, qt: f"DATEADD(SECOND, DATEDIFF(SECOND, 0, CAST({s} AS DATETIME)), 0)",
+                "minute": lambda s, qt: f"DATEADD(MINUTE, DATEDIFF(MINUTE, 0, CAST({s} AS DATETIME)), 0)",
+                "hour": lambda s, qt: f"DATEADD(HOUR, DATEDIFF(HOUR, 0, CAST({s} AS DATETIME)), 0)",
+                "date": lambda s, qt: f"CAST(CAST({s} AS DATE) AS DATETIME)",
+                "week": self._week_dimension_group_time_sql,
+                "month": lambda s, qt: f"DATEADD(MONTH, DATEDIFF(MONTH, 0, CAST({s} AS DATE)), 0)",
+                "quarter": lambda s, qt: f"DATEADD(QUARTER, DATEDIFF(QUARTER, 0, CAST({s} AS DATE)), 0)",
+                "year": lambda s, qt: f"DATEADD(YEAR, DATEDIFF(YEAR, 0, CAST({s} AS DATE)), 0)",
+                "week_index": lambda s, qt: f"EXTRACT(WEEK FROM {s})",
+                "month_of_year_index": lambda s, qt: f"EXTRACT(MONTH FROM {s})",
+                "month_of_year": lambda s, qt: f"LEFT(DATENAME(MONTH, CAST({s} AS DATE)), 3)",
+                "hour_of_day": lambda s, qt: f"DATEPART(HOUR, CAST({s} AS DATETIME))",
+                "day_of_week": lambda s, qt: f"LEFT(DATENAME(WEEKDAY, CAST({s} AS DATE)), 3)",
+                "day_of_month": lambda s, qt: f"DATEPART(DAY, CAST({s} AS DATE))",
+            },
             Definitions.bigquery: {
                 "raw": lambda s, qt: s,
                 "time": lambda s, qt: f"CAST({s} AS TIMESTAMP)",
+                "second": lambda s, qt: f"CAST(DATETIME_TRUNC(CAST({s} AS DATETIME), SECOND) AS {self.datatype.upper()})",  # noqa
+                "minute": lambda s, qt: f"CAST(DATETIME_TRUNC(CAST({s} AS DATETIME), MINUTE) AS {self.datatype.upper()})",  # noqa
+                "hour": lambda s, qt: f"CAST(DATETIME_TRUNC(CAST({s} AS DATETIME), HOUR) AS {self.datatype.upper()})",  # noqa
                 "date": lambda s, qt: f"CAST(DATE_TRUNC(CAST({s} AS DATE), DAY) AS {self.datatype.upper()})",
                 "week": self._week_dimension_group_time_sql,
                 "month": lambda s, qt: f"CAST(DATE_TRUNC(CAST({s} AS DATE), MONTH) AS {self.datatype.upper()})",  # noqa
                 "quarter": lambda s, qt: f"CAST(DATE_TRUNC(CAST({s} AS DATE), QUARTER) AS {self.datatype.upper()})",  # noqa
                 "year": lambda s, qt: f"CAST(DATE_TRUNC(CAST({s} AS DATE), YEAR) AS {self.datatype.upper()})",
+                "week_index": lambda s, qt: f"EXTRACT(WEEK FROM {s})",
+                "month_of_year_index": lambda s, qt: f"EXTRACT(MONTH FROM {s})",
                 "month_of_year": lambda s, qt: f"FORMAT_DATETIME('%B', CAST({s} as DATETIME))",
                 "hour_of_day": lambda s, qt: f"CAST({s} AS STRING FORMAT 'HH24')",
                 "day_of_week": lambda s, qt: f"CAST({s} AS STRING FORMAT 'DAY')",
@@ -606,6 +713,13 @@ class Field(MetricsLayerBase, SQLReplacement):
         }
         # Snowflake and redshift have identical syntax in this case
         meta_lookup[Definitions.redshift] = meta_lookup[Definitions.snowflake]
+        # Snowflake and duck db have identical syntax in this case
+        meta_lookup[Definitions.duck_db] = meta_lookup[Definitions.postgres]
+
+        # We alias month_name as the same thing as month_of_year to aid with looker migration
+        for _, lookup in meta_lookup.items():
+            lookup["month_name"] = lookup["month_of_year"]
+            lookup["month_index"] = lookup["month_of_year_index"]
 
         if self.view.project.timezone and self.convert_timezone:
             sql = self._apply_timezone_to_sql(sql, self.view.project.timezone, query_type)
@@ -620,10 +734,13 @@ class Field(MetricsLayerBase, SQLReplacement):
             return f"CAST(DATETIME(CAST({sql} AS TIMESTAMP), '{timezone}') AS {self.datatype.upper()})"
         elif query_type == Definitions.redshift:
             return f"CAST(CAST(CONVERT_TIMEZONE('{timezone}', {sql}) AS TIMESTAMP) AS {self.datatype.upper()})"  # noqa
-        elif query_type == Definitions.postgres:
+        elif query_type in {Definitions.postgres, Definitions.duck_db}:
             return f"CAST(CAST({sql} AS TIMESTAMP) at time zone 'utc' at time zone '{timezone}' AS {self.datatype.upper()})"  # noqa
-        elif query_type == Definitions.druid:
-            print("Warning: Druid does not support timezone conversion. Timezone will be ignored.")
+        elif query_type in {Definitions.druid, Definitions.sql_server}:
+            print(
+                f"Warning: {query_type.title()} does not support timezone conversion. "
+                "Timezone will be ignored."
+            )
             return sql
         else:
             raise QueryError(f"Unable to apply timezone to sql for query type {query_type}")
@@ -654,7 +771,7 @@ class Field(MetricsLayerBase, SQLReplacement):
             if offset is None:
                 return f"DATE_TRUNC('WEEK', {casted})"
             return f"DATE_TRUNC('WEEK', {casted} + {offset}) - {offset}"
-        elif query_type in {Definitions.postgres, Definitions.druid}:
+        elif query_type in {Definitions.postgres, Definitions.druid, Definitions.duck_db}:
             if offset is None:
                 return f"DATE_TRUNC('WEEK', CAST({sql} AS TIMESTAMP))"
             return f"DATE_TRUNC('WEEK', CAST({sql} AS TIMESTAMP) + INTERVAL '{offset}' DAY) - INTERVAL '{offset}' DAY"  # noqa
@@ -662,6 +779,12 @@ class Field(MetricsLayerBase, SQLReplacement):
             if offset is None:
                 return f"DATE_TRUNC({casted}, WEEK)"
             return f"DATE_TRUNC({casted} + {offset}, WEEK) - {offset}"
+        elif Definitions.sql_server == query_type:
+            if offset is None:
+                return f"DATEADD(WEEK, DATEDIFF(WEEK, 0, {casted}), 0)"
+            return f"DATEADD(DAY, -{offset}, DATEADD(WEEK, DATEDIFF(WEEK, 0, DATEADD(DAY, {offset}, {casted})), 0))"  # noqa
+        else:
+            raise QueryError(f"Unable to find a valid method for running week with query type {query_type}")
 
     def collect_errors(self):
         errors = []
@@ -673,11 +796,33 @@ class Field(MetricsLayerBase, SQLReplacement):
                 f"Field {self.name} is of type time, but has property "
                 "intervals when it should have property timeframes"
             )
+        if self.type == "time":
+            timeframes = self._definition.get("timeframes", [])
+            if not timeframes:
+                errors.append(
+                    f"Field {self.name} is of type time and but does not have values for the timeframe property"
+                    f"Add valid timeframes (options: {VALID_TIMEFRAMES})"
+                )
+            for i in timeframes:
+                if i not in VALID_TIMEFRAMES:
+                    errors.append(
+                        f"Field {self.name} is of type time and has timeframe value of '{i}' "
+                        f"which is not a valid timeframes (valid timeframes are {VALID_TIMEFRAMES})"
+                    )
+
         if self.type == "duration" and "timeframes" in self._definition:
             errors.append(
                 f"Field {self.name} is of type duration, but has property "
                 "timeframes when it should have property intervals"
             )
+        if self.type == "duration":
+            intervals = self._definition.get("intervals", [])
+            for i in intervals:
+                if i not in VALID_INTERVALS:
+                    errors.append(
+                        f"Field {self.name} is of type duration and has interval value of '{i}' "
+                        f"which is not a valid interval (valid intervals are {VALID_INTERVALS})"
+                    )
         if self.field_type == "measure" and not self.canon_date:
             if self.is_merged_result:
                 error_text = (
@@ -710,6 +855,13 @@ class Field(MetricsLayerBase, SQLReplacement):
             )
             errors.append(error_text)
 
+        if self.get_referenced_sql_query(strings_only=False) is None:
+            error_text = (
+                f"Field {self.view.name}.{self.name} contains invalid SQL for Zenlytic. "
+                "Remove any Looker parameter references from the SQL."
+            )
+            errors.append(error_text)
+
         if self.filters:
             for f in self.filters:
                 if any(k not in f for k in ["field", "value"]):
@@ -766,7 +918,9 @@ class Field(MetricsLayerBase, SQLReplacement):
                 to_replace_type = None if field is None else field.type
 
                 if to_replace_type == "number":
-                    reference_fields.extend(field.get_referenced_sql_query(strings_only=False))
+                    reference_fields_raw = field.get_referenced_sql_query(strings_only=False)
+                    if reference_fields_raw is not None:
+                        reference_fields.extend(reference_fields_raw)
                 elif to_replace_type is None and field is None:
                     reference_fields.append(to_replace)
                 else:
