@@ -4,6 +4,7 @@ from metrics_layer.core.exceptions import QueryError
 from collections import defaultdict
 from copy import deepcopy
 
+from metrics_layer.core.model.definitions import Definitions
 from metrics_layer.core.sql.query_merged_results import MetricsLayerMergedResultsQuery
 from metrics_layer.core.sql.single_query_resolve import SingleSQLQueryResolver
 
@@ -78,6 +79,12 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
 
         readable_metrics = {join_hash_readability_lookup[k]: m for k, m in self.query_metrics.items()}
         readable_dimensions = {join_hash_readability_lookup[k]: d for k, d in self.query_dimensions.items()}
+        readable_mapping_lookup = {
+            k: [
+                {"cte": join_hash_readability_lookup[v["from_join_hash"]], "field": v["field"]} for v in items
+            ]
+            for k, items in self.mapping_lookup.items()
+        }
         query_config = {
             "merged_metrics": self.merged_metrics,
             "query_metrics": readable_metrics,
@@ -85,10 +92,15 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
             "having": self.having,
             "queries_to_join": queries_to_join,
             "join_hashes": list(sorted(readable_join_hashes)),
+            "mapping_lookup": readable_mapping_lookup,
             "query_type": resolver.query_type,
             "limit": self.limit,
             "project": self.project,
         }
+        # Druid does not allow semicolons
+        if resolver.query_type == Definitions.druid:
+            semicolon = False
+
         merged_result_query = MetricsLayerMergedResultsQuery(query_config)
         query = merged_result_query.get_query(semicolon=semicolon)
 
@@ -99,6 +111,7 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
         self.merged_metrics = []
         self.secondary_metrics = []
         self.dimension_fields = []
+        self.mapping_lookup = {}
 
         for metric in self.metrics:
             field = self.project.get_field(metric)
@@ -134,7 +147,7 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
                 )
             key = self._cte_name_from_parts(field.canon_date, join_group_hash)
             if key in self.query_metrics:
-                already_in_query = any(field.id() in f.id() for f in self.query_metrics[key])
+                already_in_query = any(field.id() == f.id() for f in self.query_metrics[key])
                 if not already_in_query:
                     self.query_metrics[key].append(field)
             else:
@@ -143,11 +156,15 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
         self.query_metrics = self.deduplicate_fields(self.query_metrics)
 
         dimension_mapping, canon_dates, used_join_hashes = self._canon_date_mapping()
+        self.mapping_lookup = deepcopy(dimension_mapping)
 
         mappings = self.model.get_mappings(dimensions_only=True)
         for key, map_to in mappings.items():
             for other_join_hash in used_join_hashes:
-                if map_to["to_join_hash"] in other_join_hash:
+                if self._join_hash_contains_join_graph(other_join_hash, [map_to["to_join_hash"]]):
+                    self.mapping_lookup[key].append(
+                        {"field": map_to["field"], "from_join_hash": other_join_hash}
+                    )
                     map_to["from_join_hash"] = other_join_hash
                 dimension_mapping[key].append(deepcopy(map_to))
 
@@ -168,8 +185,8 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
             else:
                 not_in_metrics = True
                 for join_hash in used_join_hashes:
-                    # If the dimension is available in the join subgraph as the metric, attach it
-                    if any(jg in join_hash for jg in field.join_graphs()):
+                    # If the dimension is available in the same join subgraph as the metric, attach it
+                    if self._join_hash_contains_join_graph(join_hash, field.join_graphs()):
                         self.query_dimensions[join_hash].append(field)
                         not_in_metrics = False
                     else:
@@ -229,7 +246,6 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
                 ]
                 joinable_not_canon_date = not is_canon_date and join_group_hash in joinable_subqueries
                 is_canon_date_same = is_canon_date and join_hash_with_canon_date in join_hash
-
                 if joinable_not_canon_date or is_canon_date_same:
                     self.query_where[join_hash].append(where)
                     added_filter[join_hash] = True
@@ -273,11 +289,23 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
         joinable_sets = []
         for field in self.secondary_metrics + self.dimension_fields:
             joinable_graphs = [j for j in field.join_graphs() if "merged_result" not in j]
-            joinable = all(any(j in join_set for j in joinable_graphs) for join_set in joinable_sets)
+            all_joinable = all(any(j in join_set for j in joinable_graphs) for join_set in joinable_sets)
+            any_joinable = any(any(j in join_set for j in joinable_graphs) for join_set in joinable_sets)
 
-            cannot_join = not joinable or len(joinable_sets) == 0
-            measure_date_missing = field.canon_date not in canon_dates and field.field_type == "measure"
-            if field.canon_date and (cannot_join or measure_date_missing):
+            is_not_measure = field.field_type != "measure"
+            is_measure = field.field_type == "measure"
+            cannot_join_to_all = not all_joinable or len(joinable_sets) == 0
+            measure_date_missing = field.canon_date not in canon_dates and is_measure
+            if len(self.secondary_metrics + self.merged_metrics) > 0:
+                measure_cannot_join_to_all = cannot_join_to_all and is_measure
+                dim_cannot_join_to_any = is_not_measure and not any_joinable
+            else:
+                measure_cannot_join_to_all = cannot_join_to_all
+                dim_cannot_join_to_any = is_not_measure and not any_joinable
+
+            if field.canon_date and (
+                measure_cannot_join_to_all or dim_cannot_join_to_any or measure_date_missing
+            ):
                 canon_dates.append(field.canon_date)
                 joinable_sets.append(joinable_graphs)
 
@@ -298,14 +326,29 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
         )
         return dimension_mapping, canon_dates, sorted_joins
 
-    @staticmethod
-    def _join_hash_key(field):
+    def _join_hash_key(self, field):
         # This makes the join hash out of all the joinable graphs, it's crucial to use this when
         # naming the CTEs so in subsequent checks we can match up all joinable fields,
-        # not just fields in the same view
+        # not just fields in the same view.
+
+        # Here we're checking if the field is a measure, if so we need to get the canon date
+        # to calculate the join hash. This solves the issue where the canon date has a different
+        # join hash from its associated metric. In that case we use the canon date's join hash.
+        if field.field_type == "measure" and field.canon_date is not None:
+            field = self.project.get_field_by_name(field.canon_date)
         joinable_graphs = [jg for jg in field.join_graphs() if "merged_result" not in jg]
         return "_".join(joinable_graphs)
 
+    def _join_hash_contains_join_graph(self, join_hash: str, join_graphs: list):
+        # Due to situations like subquery_1 and subquery_12 we have to split these
+        # apart and check if any of the join graphs are in the split list
+        join_hash_subqueries = []
+        for query_number in join_hash.split("subquery"):
+            clean_query_number = query_number.strip("_")
+            if clean_query_number.isdigit():
+                join_hash_subqueries.append(f"subquery_{clean_query_number}")
+        return any(jg in join_hash_subqueries for jg in join_graphs)
+
     @staticmethod
     def _cte_name_from_parts(field_id: str, join_group_hash: str):
         canon_date = field_id.replace(".", "_")
